<?php

namespace Drupal\trash\EventSubscriber;

use Drupal\Core\Config\ConfigCrudEvent;
use Drupal\Core\Config\ConfigEvents;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\trash\TrashManagerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Listens to the config save event for trash.settings.
 */
class TrashConfigSubscriber implements EventSubscriberInterface {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The Trash manager.
   *
   * @var \Drupal\trash\TrashManagerInterface
   */
  protected $trashManager;

  /**
   * Constructs the TrashConfigSubscriber.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\trash\TrashManagerInterface $trash_manager
   *   The trash manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, TrashManagerInterface $trash_manager) {
    $this->entityTypeManager = $entity_type_manager;
    $this->trashManager = $trash_manager;
  }

  /**
   * Updates entity type definitions and ensures routes are rebuilt when needed.
   *
   * @param \Drupal\Core\Config\ConfigCrudEvent $event
   *   The ConfigCrudEvent to process.
   */
  public function onSave(ConfigCrudEvent $event) {
    $entity_types = $this->entityTypeManager->getDefinitions();
    if ($event->getConfig()->getName() === 'trash.settings' && $event->isChanged('enabled_entity_types')) {
      $enabled_entity_types = $event->getConfig()->get('enabled_entity_types');
      $original_enabled_entity_types = $event->getConfig()->getOriginal('enabled_entity_types') ?? [];

      foreach ($enabled_entity_types as $entity_type_id) {
        if (!in_array($entity_type_id, $original_enabled_entity_types, TRUE)) {
          $this->trashManager->enableEntityType($entity_types[$entity_type_id]);
        }
      }

      foreach ($original_enabled_entity_types as $entity_type_id) {
        if (!in_array($entity_type_id, $enabled_entity_types, TRUE)) {
          $this->trashManager->disableEntityType($entity_types[$entity_type_id]);
        }
      }

      // When an entity type is enabled or disabled, the router needs to be
      // rebuilt to add the corresponding tabs in the trash UI.
      \Drupal::service('router.builder')->setRebuildNeeded();
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[ConfigEvents::SAVE][] = ['onSave'];
    return $events;
  }

}
