<?php

namespace Drupal\trash;

use Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\DefaultPluginManager;
use Symfony\Component\DependencyInjection\ContainerAwareInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides custom storage handlers for trash-supported entity types.
 */
class TrashEntityTypeManager extends DefaultPluginManager implements EntityTypeManagerInterface, ContainerAwareInterface {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Contains instantiated storage handlers keyed by entity type.
   *
   * @var array
   */
  protected $storageHandlers = [];

  /**
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function setContainer(ContainerInterface $container = NULL) {
    $this->entityTypeManager->setContainer($container);
  }

  /**
   * {@inheritdoc}
   */
  public function processDefinition(&$definition, $plugin_id) {
    $this->entityTypeManager->processDefinition($definition, $plugin_id);
  }

  /**
   * {@inheritdoc}
   */
  protected function findDefinitions() {
    return $this->entityTypeManager->findDefinitions();
  }

  /**
   * {@inheritdoc}
   */
  public function getDefinition($entity_type_id, $exception_on_invalid = TRUE) {
    return $this->entityTypeManager->getDefinition($entity_type_id, $exception_on_invalid);
  }

  /**
   * {@inheritdoc}
   */
  public function getActiveDefinition($entity_type_id) {
    return $this->entityTypeManager->getActiveDefinition($entity_type_id);
  }

  /**
   * {@inheritdoc}
   */
  public function clearCachedDefinitions() {
    $this->entityTypeManager->clearCachedDefinitions();
    $this->storageHandlers = [];
  }

  /**
   * {@inheritdoc}
   */
  public function useCaches($use_caches = FALSE) {
    $this->entityTypeManager->useCaches($use_caches);
    if (!$use_caches) {
      $this->storageHandlers = [];
    }
  }

  /**
   * {@inheritdoc}
   */
  public function hasHandler($entity_type_id, $handler_type) {
    return $this->entityTypeManager->hasHandler($entity_type_id, $handler_type);
  }

  /**
   * {@inheritdoc}
   */
  public function getStorage($entity_type_id) {
    return $this->getHandler($entity_type_id, 'storage');
  }

  /**
   * {@inheritdoc}
   */
  public function getListBuilder($entity_type_id) {
    return $this->entityTypeManager->getListBuilder($entity_type_id);
  }

  /**
   * {@inheritdoc}
   */
  public function getFormObject($entity_type_id, $operation) {
    return $this->entityTypeManager->getFormObject($entity_type_id, $operation);
  }

  /**
   * {@inheritdoc}
   */
  public function getRouteProviders($entity_type_id) {
    return $this->entityTypeManager->getRouteProviders($entity_type_id);
  }

  /**
   * {@inheritdoc}
   */
  public function getViewBuilder($entity_type_id) {
    return $this->entityTypeManager->getViewBuilder($entity_type_id);
  }

  /**
   * {@inheritdoc}
   */
  public function getAccessControlHandler($entity_type_id) {
    return $this->entityTypeManager->getAccessControlHandler($entity_type_id);
  }

  /**
   * {@inheritdoc}
   */
  public function getHandler($entity_type_id, $handler_type) {
    if ($handler_type !== 'storage') {
      return $this->entityTypeManager->getHandler($entity_type_id, $handler_type);
    }

    if (!isset($this->storageHandlers[$entity_type_id])) {
      $definition = $this->getDefinition($entity_type_id);
      $class = $definition->getHandlerClass($handler_type);
      if (!$class) {
        throw new InvalidPluginDefinitionException($entity_type_id, sprintf('The "%s" entity type did not specify a %s handler.', $entity_type_id, $handler_type));
      }
      $this->storageHandlers[$entity_type_id] = $this->createHandlerInstance($class, $definition);
    }

    return $this->storageHandlers[$entity_type_id];
  }

  /**
   * {@inheritdoc}
   */
  public function createHandlerInstance($class, EntityTypeInterface $definition = NULL) {
    if (\Drupal::service('trash.manager')->isEntityTypeEnabled($definition)) {
      $class = _trash_generate_storage_class($class);
    }

    return $this->entityTypeManager->createHandlerInstance($class, $definition);
  }

}
